;
; keyb.asm
;
; Keyboard code
;

extern __go32_dpmi_lock_data
extern __go32_dpmi_lock_code

bits 32
section .data

times ($$-$) & 3 nop
data_start:
    input_old_selector  dd 0    ; selector for old keyboard driver
    input_old_offset    dd 0    ; offset for old keyboard driver
    key                 dd 0
data_end:

section .text

times ($$-$) & 3 nop
code_start:
    ;
    ; KeybInit() saves our DS here. This is required to prevent the interrupt
    ; handler from crashing things.
    ;
    our_ds  dd 0
KeyboardDriver:
    ;
    ; This is our keyboard driver. It places key-presses into the buffer.
    ; Key-releases are ignored.
    ;

    cli
    push    ds
    push    eax

    mov     ds, [cs:our_ds]
                      
    in      al, 0x60
    mov     [key], al
.end:
    mov     al, 0x20    ; issue EOI
    out     0x20, al

    pop     eax
    pop     ds
    sti
    iret
code_end:

;
; UINT8 KeybGetKey();
;
; Returns the scan code of the last key pressed and then clears the buffer.
;

times ($$-$) & 3 nop
global _KeybGetKey
_KeybGetKey:
    xor     eax, eax
    xchg    al, [key]
    ret

;
; BOOL KeybInit();
;
; Installs the keyboard driver. Returns non-zero if failed.
;
times ($$-$) & 3 nop
global _KeybInit
_KeybInit:
    mov     [our_ds], ds ; save the DS

    pushad
    xor     eax, eax
    
    ;
    ; Lock data and code areas that driver uses
    ;
    push    dword (data_end - data_start)
    push    dword data_start
    call    __go32_dpmi_lock_data
    add     esp, (2*4)
    test    eax, eax
    jnz     short .error

    push    dword (code_end - code_start)
    push    dword code_start
    call    __go32_dpmi_lock_code
    add     esp, (2*4)
    test    eax, eax
    jnz     short .error

    ;
    ; Get vector of old protected mode keyboard driver
    ;
    mov     ax, 0x204
    mov     bl, 9
    int     0x31
    mov     [input_old_selector], cx
    mov     [input_old_offset], edx

    ;
    ; Install keyboard driver
    ;
    mov     ax, 0x205
    mov     bl, 9
    mov     cx, cs
    mov     edx, KeyboardDriver
    int     0x31
    jc      short .error

    popad
    xor     eax, eax
    ret

.error:
    popad
    mov     eax, 1
    ret

;
; void KeybDeinit();
;
; Uninstalls the keyboard driver and hooks the old one back up.
;

times ($$-$) & 3 nop
global _KeybDeinit
_KeybDeinit:
    pushad
    mov     ax, 0x205
    mov     bl, 9
    mov     cx, [input_old_selector]
    mov     edx, [input_old_offset]
    int     0x31
    popad
    ret
